@@ -294,6 +294,7 @@ names are also used in Bio.AlignIO and include the following:
       which encodes PHRED quality scores with an ASCII offset of 64
       (not 33). Note as of version 1.8 of the CASAVA pipeline Illumina
       will produce FASTQ files using the standard Sanger encoding.
+    - gck     - Gene Construction Kit's format.
     - genbank - The GenBank or GenPept flat file format.
     - gb      - An alias for "genbank", for consistency with NCBI Entrez Utilities
     - ig      - The IntelliGenetics file format, apparently the same as the
@@ -301,8 +302,8 @@ names are also used in Bio.AlignIO and include the following:
     - imgt    - An EMBL like format from IMGT where the feature tables are more
       indented to allow for longer feature types.
     - nib     - UCSC's nib file format for nucleotide sequences, which uses one
-                nibble (4 bits) to represent each nucleotide, and stores two
-                nucleotides in one byte.
+      nibble (4 bits) to represent each nucleotide, and stores two nucleotides in
+      one byte.
     - pdb-seqres -  Reads a Protein Data Bank (PDB) file to determine the
       complete protein sequence as it appears in the header (no dependencies).
     - pdb-atom - Uses Bio.PDB to determine the (partial) protein sequence as
@@ -315,6 +316,7 @@ names are also used in Bio.AlignIO and include the following:
     - seqxml  - SeqXML, simple XML format described in Schmitt et al (2011).
     - sff     - Standard Flowgram Format (SFF), typical output from Roche 454.
     - sff-trim - Standard Flowgram Format (SFF) with given trimming applied.
+    - snapgene - SnapGene's native format.
     - swiss   - Plain text Swiss-Prot aka UniProt format.
     - tab     - Simple two column tab separated sequence files, where each
       line holds a record's identifier and sequence. For example,
@@ -325,6 +327,7 @@ names are also used in Bio.AlignIO and include the following:
       in separate FASTA files).
     - uniprot-xml - The UniProt XML format (replacement for the SwissProt plain
       text format which we call "swiss")
+    - xdna        - DNA Strider's and SerialCloner's native format.
 
 Note that while Bio.SeqIO can read all the above file formats, it cannot
 write to all of them.
@@ -390,6 +393,7 @@ from Bio.Alphabet import Alphabet, AlphabetEncoder, _get_base_alphabet
 from . import AbiIO
 from . import AceIO
 from . import FastaIO
+from . import GckIO
 from . import IgIO  # IntelliGenetics or MASE format
 from . import InsdcIO  # EMBL and GenBank
 from . import NibIO
@@ -398,10 +402,12 @@ from . import PhdIO
 from . import PirIO
 from . import SeqXmlIO
 from . import SffIO
+from . import SnapGeneIO
 from . import SwissIO
 from . import TabIO
 from . import QualityIO  # FastQ and qual files
 from . import UniprotIO
+from . import XdnaIO
 
 if sys.version_info < (3, 6):
     from collections import OrderedDict as _dict
@@ -418,38 +424,42 @@ else:
 #
 # Most alignment file formats will be handled via Bio.AlignIO
 
-_FormatToIterator = {"abi": AbiIO.AbiIterator,
-                     "abi-trim": AbiIO._AbiTrimIterator,
-                     "ace": AceIO.AceIterator,
-                     "fasta": FastaIO.FastaIterator,
-                     "fasta-2line": FastaIO.FastaTwoLineIterator,
-                     "ig": IgIO.IgIterator,
-                     "embl": InsdcIO.EmblIterator,
-                     "embl-cds": InsdcIO.EmblCdsFeatureIterator,
-                     "gb": InsdcIO.GenBankIterator,
-                     "genbank": InsdcIO.GenBankIterator,
-                     "genbank-cds": InsdcIO.GenBankCdsFeatureIterator,
-                     "imgt": InsdcIO.ImgtIterator,
-                     "nib": NibIO.NibIterator,
-                     "cif-seqres": PdbIO.CifSeqresIterator,
-                     "cif-atom": PdbIO.CifAtomIterator,
-                     "pdb-atom": PdbIO.PdbAtomIterator,
-                     "pdb-seqres": PdbIO.PdbSeqresIterator,
-                     "phd": PhdIO.PhdIterator,
-                     "pir": PirIO.PirIterator,
-                     "fastq": QualityIO.FastqPhredIterator,
-                     "fastq-sanger": QualityIO.FastqPhredIterator,
-                     "fastq-solexa": QualityIO.FastqSolexaIterator,
-                     "fastq-illumina": QualityIO.FastqIlluminaIterator,
-                     "qual": QualityIO.QualPhredIterator,
-                     "seqxml": SeqXmlIO.SeqXmlIterator,
-                     "sff": SffIO.SffIterator,
-                     # Not sure about this in the long run:
-                     "sff-trim": SffIO._SffTrimIterator,
-                     "swiss": SwissIO.SwissIterator,
-                     "tab": TabIO.TabIterator,
-                     "uniprot-xml": UniprotIO.UniprotIterator,
-                     }
+_FormatToIterator = {
+    "abi": AbiIO.AbiIterator,
+    "abi-trim": AbiIO._AbiTrimIterator,
+    "ace": AceIO.AceIterator,
+    "fasta": FastaIO.FastaIterator,
+    "fasta-2line": FastaIO.FastaTwoLineIterator,
+    "ig": IgIO.IgIterator,
+    "embl": InsdcIO.EmblIterator,
+    "embl-cds": InsdcIO.EmblCdsFeatureIterator,
+    "gb": InsdcIO.GenBankIterator,
+    "gck": GckIO.GckIterator,
+    "genbank": InsdcIO.GenBankIterator,
+    "genbank-cds": InsdcIO.GenBankCdsFeatureIterator,
+    "imgt": InsdcIO.ImgtIterator,
+    "nib": NibIO.NibIterator,
+    "cif-seqres": PdbIO.CifSeqresIterator,
+    "cif-atom": PdbIO.CifAtomIterator,
+    "pdb-atom": PdbIO.PdbAtomIterator,
+    "pdb-seqres": PdbIO.PdbSeqresIterator,
+    "phd": PhdIO.PhdIterator,
+    "pir": PirIO.PirIterator,
+    "fastq": QualityIO.FastqPhredIterator,
+    "fastq-sanger": QualityIO.FastqPhredIterator,
+    "fastq-solexa": QualityIO.FastqSolexaIterator,
+    "fastq-illumina": QualityIO.FastqIlluminaIterator,
+    "qual": QualityIO.QualPhredIterator,
+    "seqxml": SeqXmlIO.SeqXmlIterator,
+    "sff": SffIO.SffIterator,
+    "snapgene": SnapGeneIO.SnapGeneIterator,
+    # Not sure about this in the long run:
+    "sff-trim": SffIO._SffTrimIterator,
+    "swiss": SwissIO.SwissIterator,
+    "tab": TabIO.TabIterator,
+    "uniprot-xml": UniprotIO.UniprotIterator,
+    "xdna": XdnaIO.XdnaIterator,
+}
 
 _FormatToString = {
     "fasta": FastaIO.as_fasta,
@@ -464,26 +474,38 @@ _FormatToString = {
 
 # This could exclude file formats covered by _FormatToString?
 # Right now used in the unit tests as proxy for all supported outputs...
-_FormatToWriter = {"fasta": FastaIO.FastaWriter,
-                   "fasta-2line": FastaIO.FastaTwoLineWriter,
-                   "gb": InsdcIO.GenBankWriter,
-                   "genbank": InsdcIO.GenBankWriter,
-                   "embl": InsdcIO.EmblWriter,
-                   "imgt": InsdcIO.ImgtWriter,
-                   "nib": NibIO.NibWriter,
-                   "phd": PhdIO.PhdWriter,
-                   "pir": PirIO.PirWriter,
-                   "fastq": QualityIO.FastqPhredWriter,
-                   "fastq-sanger": QualityIO.FastqPhredWriter,
-                   "fastq-solexa": QualityIO.FastqSolexaWriter,
-                   "fastq-illumina": QualityIO.FastqIlluminaWriter,
-                   "qual": QualityIO.QualPhredWriter,
-                   "seqxml": SeqXmlIO.SeqXmlWriter,
-                   "sff": SffIO.SffWriter,
-                   "tab": TabIO.TabWriter,
-                   }
+_FormatToWriter = {
+    "fasta": FastaIO.FastaWriter,
+    "fasta-2line": FastaIO.FastaTwoLineWriter,
+    "gb": InsdcIO.GenBankWriter,
+    "genbank": InsdcIO.GenBankWriter,
+    "embl": InsdcIO.EmblWriter,
+    "imgt": InsdcIO.ImgtWriter,
+    "nib": NibIO.NibWriter,
+    "phd": PhdIO.PhdWriter,
+    "pir": PirIO.PirWriter,
+    "fastq": QualityIO.FastqPhredWriter,
+    "fastq-sanger": QualityIO.FastqPhredWriter,
+    "fastq-solexa": QualityIO.FastqSolexaWriter,
+    "fastq-illumina": QualityIO.FastqIlluminaWriter,
+    "qual": QualityIO.QualPhredWriter,
+    "seqxml": SeqXmlIO.SeqXmlWriter,
+    "sff": SffIO.SffWriter,
+    "tab": TabIO.TabWriter,
+    "xdna": XdnaIO.XdnaWriter,
+}
 
-_BinaryFormats = ["sff", "sff-trim", "abi", "abi-trim", "seqxml", "nib"]
+_BinaryFormats = [
+    "sff",
+    "sff-trim",
+    "abi",
+    "abi-trim",
+    "gck",
+    "seqxml",
+    "snapgene",
+    "nib",
+    "xdna",
+]
 
 
 def write(sequences, handle, format):
@@ -522,9 +544,9 @@ def write(sequences, handle, format):
         sequences = [sequences]
 
     if format in _BinaryFormats:
-        mode = 'wb'
+        mode = "wb"
     else:
-        mode = 'w'
+        mode = "w"
 
     with as_handle(handle, mode) as fp:
         # Map the file format to a writer function/class
@@ -543,21 +565,24 @@ def write(sequences, handle, format):
             alignment = MultipleSeqAlignment(sequences)
             alignment_count = AlignIO.write([alignment], fp, format)
             if alignment_count != 1:
-                raise RuntimeError("Internal error - the underlying writer "
-                                   "should have returned 1, not %r"
-                                   % alignment_count)
+                raise RuntimeError(
+                    "Internal error - the underlying writer "
+                    "should have returned 1, not %r" % alignment_count
+                )
             count = len(alignment)
             del alignment_count, alignment
         elif format in _FormatToIterator or format in AlignIO._FormatToIterator:
-            raise ValueError("Reading format '%s' is supported, but not writing"
-                             % format)
+            raise ValueError(
+                "Reading format '%s' is supported, but not writing" % format
+            )
         else:
             raise ValueError("Unknown format '%s'" % format)
 
         if not isinstance(count, int):
-            raise RuntimeError("Internal error - the underlying %s writer "
-                               "should have returned the record count, not %r"
-                               % (format, count))
+            raise RuntimeError(
+                "Internal error - the underlying %s writer "
+                "should have returned the record count, not %r" % (format, count)
+            )
 
     return count
 
@@ -624,9 +649,9 @@ def parse(handle, format, alphabet=None):
 
     # Hack for SFF, will need to make this more general in future
     if format in _BinaryFormats:
-        mode = 'rb'
+        mode = "rb"
     else:
-        mode = 'rU'
+        mode = "rU"
 
     # Try and give helpful error messages:
     if not isinstance(format, basestring):
@@ -635,8 +660,9 @@ def parse(handle, format, alphabet=None):
         raise ValueError("Format required (lower case string)")
     if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
-    if alphabet is not None and not (isinstance(alphabet, Alphabet) or
-                                     isinstance(alphabet, AlphabetEncoder)):
+    if alphabet is not None and not (
+        isinstance(alphabet, Alphabet) or isinstance(alphabet, AlphabetEncoder)
+    ):
         raise ValueError("Invalid alphabet, %r" % alphabet)
 
     with as_handle(handle, mode) as fp:
@@ -652,9 +678,11 @@ def parse(handle, format, alphabet=None):
                     i = _force_alphabet(iterator_generator(fp), alphabet)
         elif format in AlignIO._FormatToIterator:
             # Use Bio.AlignIO to read in the alignments
-            i = (r for alignment in AlignIO.parse(fp, format,
-                                                  alphabet=alphabet)
-                 for r in alignment)
+            i = (
+                r
+                for alignment in AlignIO.parse(fp, format, alphabet=alphabet)
+                for r in alignment
+            )
         else:
             raise ValueError("Unknown format '%s'" % format)
         # This imposes some overhead... wait until we drop Python 2.4 to fix it
@@ -667,14 +695,14 @@ def _force_alphabet(record_iterator, alphabet):
     # Assume the alphabet argument has been pre-validated
     given_base_class = _get_base_alphabet(alphabet).__class__
     for record in record_iterator:
-        if isinstance(_get_base_alphabet(record.seq.alphabet),
-                      given_base_class):
+        if isinstance(_get_base_alphabet(record.seq.alphabet), given_base_class):
             record.seq.alphabet = alphabet
             yield record
         else:
-            raise ValueError("Specified alphabet %r clashes with "
-                             "that determined from the file, %r"
-                             % (alphabet, record.seq.alphabet))
+            raise ValueError(
+                "Specified alphabet %r clashes with "
+                "that determined from the file, %r" % (alphabet, record.seq.alphabet)
+            )
 
 
 def read(handle, format, alphabet=None):
@@ -800,6 +828,8 @@ def to_dict(sequences, key_function=None):
     Biopython 1.72, on older versions of Python we explicitly use an
     OrderedDict so that you can always assume the record order is preserved.
     """
+    # This is to avoid a lambda function:
+
     def _default_key_function(rec):
         return rec.id
 
@@ -937,25 +967,33 @@ def index(filename, format, alphabet=None, key_function=None):
         raise ValueError("Format required (lower case string)")
     if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
-    if alphabet is not None and not (isinstance(alphabet, Alphabet) or
-                                     isinstance(alphabet, AlphabetEncoder)):
+    if alphabet is not None and not (
+        isinstance(alphabet, Alphabet) or isinstance(alphabet, AlphabetEncoder)
+    ):
         raise ValueError("Invalid alphabet, %r" % alphabet)
 
     # Map the file format to a sequence iterator:
     from ._index import _FormatToRandomAccess  # Lazy import
     from Bio.File import _IndexedSeqFileDict
+
     try:
         proxy_class = _FormatToRandomAccess[format]
     except KeyError:
         raise ValueError("Unsupported format %r" % format)
-    repr = "SeqIO.index(%r, %r, alphabet=%r, key_function=%r)" \
-        % (filename, format, alphabet, key_function)
-    return _IndexedSeqFileDict(proxy_class(filename, format, alphabet),
-                               key_function, repr, "SeqRecord")
+    repr = "SeqIO.index(%r, %r, alphabet=%r, key_function=%r)" % (
+        filename,
+        format,
+        alphabet,
+        key_function,
+    )
+    return _IndexedSeqFileDict(
+        proxy_class(filename, format, alphabet), key_function, repr, "SeqRecord"
+    )
 
 
-def index_db(index_filename, filenames=None, format=None, alphabet=None,
-             key_function=None):
+def index_db(
+    index_filename, filenames=None, format=None, alphabet=None, key_function=None
+):
     """Index several sequence files and return a dictionary like object.
 
     The index is stored in an SQLite database rather than in memory (as in the
@@ -1013,21 +1051,24 @@ def index_db(index_filename, filenames=None, format=None, alphabet=None,
         # to Bio.SeqIO.index(...) for indexing just one file.
         filenames = [filenames]
     if filenames is not None and not isinstance(filenames, list):
-        raise TypeError(
-            "Need a list of filenames (as strings), or one filename")
+        raise TypeError("Need a list of filenames (as strings), or one filename")
     if format is not None and not isinstance(format, basestring):
         raise TypeError("Need a string for the file format (lower case)")
     if format and format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
-    if alphabet is not None and not (isinstance(alphabet, Alphabet) or
-                                     isinstance(alphabet, AlphabetEncoder)):
+    if alphabet is not None and not (
+        isinstance(alphabet, Alphabet) or isinstance(alphabet, AlphabetEncoder)
+    ):
         raise ValueError("Invalid alphabet, %r" % alphabet)
 
     # Map the file format to a sequence iterator:
     from ._index import _FormatToRandomAccess  # Lazy import
     from Bio.File import _SQLiteManySeqFilesDict
-    repr = ("SeqIO.index_db(%r, filenames=%r, format=%r, alphabet=%r, key_function=%r)"
-            % (index_filename, filenames, format, alphabet, key_function))
+
+    repr = (
+        "SeqIO.index_db(%r, filenames=%r, format=%r, alphabet=%r, key_function=%r)"
+        % (index_filename, filenames, format, alphabet, key_function)
+    )
 
     def proxy_factory(format, filename=None):
         """Given a filename returns proxy object, else boolean if format OK."""
@@ -1036,9 +1077,9 @@ def index_db(index_filename, filenames=None, format=None, alphabet=None,
         else:
             return format in _FormatToRandomAccess
 
-    return _SQLiteManySeqFilesDict(index_filename, filenames,
-                                   proxy_factory, format,
-                                   key_function, repr)
+    return _SQLiteManySeqFilesDict(
+        index_filename, filenames, proxy_factory, format, key_function, repr
+    )
 
 
 def convert(in_file, in_format, out_file, out_format, alphabet=None):
@@ -1076,23 +1117,24 @@ def convert(in_file, in_format, out_file, out_format, alphabet=None):
     <BLANKLINE>
     """
     if in_format in _BinaryFormats:
-        in_mode = 'rb'
+        in_mode = "rb"
     else:
-        in_mode = 'rU'
+        in_mode = "rU"
 
     if out_format in _BinaryFormats:
-        out_mode = 'wb'
+        out_mode = "wb"
     else:
-        out_mode = 'w'
+        out_mode = "w"
 
     # This will check the arguments and issue error messages,
     # after we have opened the file which is a shame.
     from ._convert import _handle_convert  # Lazy import
+
     with as_handle(in_file, in_mode) as in_handle:
         with as_handle(out_file, out_mode) as out_handle:
-            count = _handle_convert(in_handle, in_format,
-                                    out_handle, out_format,
-                                    alphabet)
+            count = _handle_convert(
+                in_handle, in_format, out_handle, out_format, alphabet
+            )
     return count
 
 
